@@ -46,7 +46,7 @@ class PyHocaGUI_AboutFrame(wx.Frame):
     wxWidget displaying an ,,About'' window for this application.
 
     """
-    def __init__(self, _PyHocaGUI, caller=None, about_image=None, icon_name='pyhoca-winicon.png', about_what=None, ):
+    def __init__(self, _PyHocaGUI, caller=None, about_image=None, icon_name='pyhoca-winicon', about_what=None, ):
         """\
         About window (constructor).
 
@@ -70,10 +70,15 @@ class PyHocaGUI_AboutFrame(wx.Frame):
         if about_image is None:
             about_image = fallback_about_image
 
+        if not about_image.lower().endswith('.png'):
+            about_image = '%s.png' % about_image
+
+        about_image = os.path.expanduser(about_image)
+
         if os.path.basename(about_image) == about_image:
             about_image = os.path.join(basepath.images_basepath, about_image)
 
-        if not (os.path.isfile(about_image) or os.path.islink(about_image)):
+        if not os.path.isfile(about_image):
             about_image = os.path.join(basepath.images_basepath, fallback_about_image)
 
         if about_what is None:
@@ -103,7 +108,14 @@ class PyHocaGUI_AboutFrame(wx.Frame):
         elif "wxMAC" in wx.PlatformInfo:
             icon_size = '128x128'
 
-        icon_file = os.path.normpath('%s/PyHoca/%s/%s.png' % (basepath.icons_basepath, icon_size, icon_name))
+        icon_name = os.path.expanduser(icon_name)
+        if not icon_name.lower().endswith('.png'):
+            icon_name = '%s.png' % icon_name
+
+        icon_file = icon_name
+        if not (os.path.isfile(str(icon_file)) or os.path.islink(str(icon_file))):
+            icon_file = '%s/PyHoca/%s/%s' % (basepath.icons_basepath, icon_size, icon_name)
+
         if not (os.path.isfile(str(icon_file)) or os.path.islink(str(icon_file))):
             icon_file = os.path.normpath('%s/PyHoca/%s/%s.png' % (basepath.icons_basepath, icon_size, 'pyhoca-winicon'))
 
