@@ -51,6 +51,7 @@ import locale
 # PyHoca-GUI modules
 import about
 import logon
+import passphrase
 import taskbar
 import profilemanager
 import printingprefs
@@ -546,8 +547,10 @@ class PyHocaGUI(wx.App, x2go.X2GoClient):
                     elif session_list[session_name].is_published_applications_provider() and pubapp_session_resumed:
 
                         # if there are more then one published applications mode sessions (in suspended state), terminate them now...
-
-                        connected_session.terminate(session_name=session_name)
+                        try:
+                            connected_session.terminate()
+                        except x2go.X2GoSessionException:
+                            pass
 
             if not pubapp_session_resumed:
 
@@ -608,7 +611,7 @@ class PyHocaGUI(wx.App, x2go.X2GoClient):
             _can_sshproxy_auto_connect = self._X2GoClient__session_can_sshproxy_auto_connect(session_uuid)
             _session_uses_sshproxy = self._X2GoClient__session_uses_sshproxy(session_uuid)
             _session_reuses_sshproxy_authinfo = self._X2GoClient__session_reuses_sshproxy_authinfo(session_uuid)
-            if _can_session_auto_connect:
+            if _can_session_auto_connect or _can_sshproxy_auto_connect:
                 self._X2GoClient__connect_session(session_uuid, add_to_known_hosts=self.add_to_known_hosts)
                 if not self._X2GoClient__server_valid_x2gouser(session_uuid):
                     self.notifier.send(_(u'%s - connect failure') % profile_name, _(u'User is not allowed to start X2Go sessions!'), icon='session_warning', timeout=10000)
@@ -629,6 +632,26 @@ class PyHocaGUI(wx.App, x2go.X2GoClient):
                 _logon_window = logon.PyHocaGUI_DialogBoxPassword(self, profile_name, caller=self, sshproxy_auth=((not _can_sshproxy_auto_connect) and _session_uses_sshproxy and (not _session_reuses_sshproxy_authinfo)))
                 self._logon_windows[profile_name] = _logon_window
 
+        except x2go.PasswordRequiredException:
+            key_filename = None
+            try:
+                if not self._X2GoClient__get_session(session_uuid).control_params['look_for_keys']:
+                    key_filename = self._X2GoClient__get_session(session_uuid).control_params['key_filename']
+            except KeyError:
+                pass
+            self._pyhoca_logger('SSH private key file is encrypted and requires a passphrase', loglevel=x2go.log.loglevel_INFO, )
+            _passphrase_window = passphrase.PyHocaGUI_DialogBoxPassphrase(self, profile_name, caller=self, key_filename=key_filename)
+            self._logon_windows[profile_name] = _passphrase_window
+        except x2go.X2GoSSHProxyPasswordRequiredException:
+            key_filename = None
+            try:
+                if not self._X2GoClient__get_session(session_uuid).sshproxy_params['sshproxy_look_for_keys']:
+                    key_filename = self._X2GoClient__get_session(session_uuid).sshproxy_params['sshproxy_key_filename']
+            except KeyError:
+                pass
+            self._pyhoca_logger('SSH private key file (for SSH proxy) is encrypted and requires a passphrase', loglevel=x2go.log.loglevel_INFO, )
+            _passphrase_window = passphrase.PyHocaGUI_DialogBoxPassphrase(self, profile_name, caller=self, sshproxy_auth=True, key_filename=key_filename)
+            self._logon_windows[profile_name] = _passphrase_window
         except x2go.AuthenticationException:
             self._pyhoca_logger('SSH key authentication to server failed, trying next auth-mechanism', loglevel=x2go.log.loglevel_INFO, )
             _logon_window = logon.PyHocaGUI_DialogBoxPassword(self, profile_name, caller=self, )
@@ -641,16 +664,46 @@ class PyHocaGUI(wx.App, x2go.X2GoClient):
                 _logon_window = logon.PyHocaGUI_DialogBoxPassword(self, profile_name, caller=self, sshproxy_auth=True, )
             self._logon_windows[profile_name] = _logon_window
         except x2go.SSHException, e:
-            if str(e).startswith('Host key for server ') and str(e).endswith(' does not match!'):
-                errmsg = _('Host key verification failed. The X2Go server may have been compromised.\n\nIt is also possible that the host key has just been changed.\n\nHowever, for security reasons the connection will not be established!!!')
+            if str(e).startswith('Two-factor authentication requires a password'):
+                self._pyhoca_logger('X2Go Server requests two-factor authentication', loglevel=x2go.loglevel_NOTICE)
+
+                _logon_window = logon.PyHocaGUI_DialogBoxPassword(self, profile_name,
+                                                                  caller=self,
+                                                                  sshproxy_auth=False,
+                                                                  twofactor_auth=True,
+                                                                 )
+                self._logon_windows[profile_name] = _logon_window
             else:
-                errmsg = str(e)
-            self.notifier.send(_(u'%s - SSH error') % profile_name, u'%s!' % errmsg, icon='auth_error', timeout=10000)
-            try:
-                self._temp_disabled_profile_names.remove(profile_name)
-            except ValueError:
-                pass
-            connect_failed = True
+                if str(e).startswith('Host key for server ') and str(e).endswith(' does not match!'):
+                    errmsg = _('Host key verification failed. The X2Go server may have been compromised.\n\nIt is also possible that the host key has just been changed.\n\nHowever, for security reasons the connection will not be established!!!')
+                else:
+                    errmsg = str(e)
+                try:
+                    self._temp_disabled_profile_names.remove(profile_name)
+                except ValueError:
+                    pass
+                connect_failed = True
+                self.notifier.send(_(u'%s - SSH error') % profile_name, u'%s!' % errmsg, icon='auth_error', timeout=10000)
+        except x2go.X2GoSSHProxyException, e:
+            if str(e).startswith('Two-factor authentication requires a password'):
+                self._pyhoca_logger('SSH proxy host requests two-factor authentication', loglevel=x2go.loglevel_NOTICE)
+                _logon_window = logon.PyHocaGUI_DialogBoxPassword(self, profile_name,
+                                                                  caller=self,
+                                                                  sshproxy_auth=True,
+                                                                  sshproxy_twofactor_auth=True,
+                                                                 )
+                self._logon_windows[profile_name] = _logon_window
+            else:
+                if str(e).startswith('Host key for server ') and str(e).endswith(' does not match!'):
+                    errmsg = _('Host key verification failed. The SSH proxy server may have been compromised.\n\nIt is also possible that the host key has just been changed.\n\nHowever, for security reasons the connection will not be established!!!')
+                else:
+                    errmsg = str(e)
+                try:
+                    self._temp_disabled_profile_names.remove(profile_name)
+                except ValueError:
+                    pass
+                connect_failed = True
+                self.notifier.send(_(u'%s - SSH error') % profile_name, u'%s!' % errmsg, icon='auth_error', timeout=10000)
         except x2go.X2GoHostKeyException, e:
             self.notifier.send(_(u'%s - host key error') % profile_name, _(u'The remote server\'s host key is invalid or has not been accepted by the user') + '!', icon='auth_error', timeout=4000)
             try:
@@ -686,13 +739,6 @@ class PyHocaGUI(wx.App, x2go.X2GoClient):
             except ValueError:
                 pass
             connect_failed = True
-        except x2go.X2GoSSHProxyException, e:
-            self.notifier.send(_(u'%s - auth key error') % profile_name, u'%s' % str(e), icon='auth_error', timeout=4000)
-            try:
-                self._temp_disabled_profile_names.remove(profile_name)
-            except ValueError:
-                pass
-            connect_failed = True
         except x2go.X2GoRemoteHomeException, e:
             self.notifier.send(_(u'%s - missing home directory') % profile_name, _(u"The remote user's home directory does not exist."), icon='auth_error', timeout=4000)
             try:
@@ -1109,11 +1155,8 @@ class PyHocaGUI(wx.App, x2go.X2GoClient):
         m.ShowModal()
         if m.Yes():
             self._pyhoca_logger('deleting session profile %s' % profile_name, loglevel=x2go.log.loglevel_INFO, )
-            try:
-                self.session_profiles.delete_profile(profile_name)
-                self.notifier.send(title=_(u'%s - profile deleted') % profile_name, text=_(u'The session profile has been deleted.'), icon='profile_delete')
-            except:
-                pass
+            self.session_profiles.delete_profile(profile_name)
+            self.notifier.send(title=_(u'%s - profile deleted') % profile_name, text=_(u'The session profile has been deleted.'), icon='profile_delete')
         self._temp_disabled_profile_names.remove(profile_name)
 
     def OnProfileImport(self, evt):#
@@ -1507,7 +1550,25 @@ class PyHocaGUI(wx.App, x2go.X2GoClient):
 
         """
         if not self._exiting:
-            self.notifier.send(_(u'%s - channel error') % profile_name, _(u'Lost connection to server %s unexpectedly! Try to re-authenticate to the server...') % profile_name, icon='session_warning', timeout=10000)
+            self.notifier.send(_(u'%s - channel error') % profile_name, _(u'Lost connection to server %s unexpectedly!\n\nTry to re-authenticate to the server...') % profile_name, icon='session_warning', timeout=10000)
+        try:
+            del self._temp_disabled_session_names[profile_name]
+        except KeyError:
+            pass
+        if self.exit_on_disconnect:
+            self.WakeUpIdle()
+            self.ExitMainLoop()
+
+    def HOOK_on_failing_SFTP_client(self, profile_name='UNKNOWN', **kwargs):
+        """\
+        Notify about SFTP client failures
+
+        @param profile_name: profile name of session that called this hook method
+        @type profile_name: C{str}
+
+        """
+        if not self._exiting:
+            self.notifier.send(_(u'%s - SFTP client error') % profile_name, _(u'New X2Go session will lack SFTP client support.\nCheck your server setup.\n\nAvoid echoing ~/.*shrc files on server!!!\n\nNot starting new session...'), icon='session_error', timeout=10000)
         try:
             del self._temp_disabled_session_names[profile_name]
         except KeyError:
@@ -1529,6 +1590,19 @@ class PyHocaGUI(wx.App, x2go.X2GoClient):
             self.WakeUpIdle()
             self.ExitMainLoop()
 
+    def HOOK_desktop_sharing_denied(self, profile_name='UNKNOWN', **kwargs):
+        """\
+        Notify about the denial of desktop sharing by the other user.
+
+        @param profile_name: profile name of session that called this hook method
+        @type profile_name: C{str}
+
+        """
+        self.notifier.send(_(u'%s - desktop sharing failure') % profile_name, _(u'Desktop sharing was denied by the other user or\nboth of you have insufficient privileges to share one another\'s desktop.'), icon='session_error', timeout=10000)
+        if self.exit_on_disconnect:
+            self.WakeUpIdle()
+            self.ExitMainLoop()
+
     def HOOK_list_desktops_timeout(self, profile_name='UNKNOWN', **kwargs):
         """\
         Notify about x2golistdesktops timeout events.
