@@ -40,7 +40,6 @@ gevent.monkey.patch_all()
 
 import wx
 import os
-import wx.lib.mixins.listctrl as listmix
 
 # PyHoca-GUI modules
 # ... NONE ...
@@ -99,6 +98,7 @@ class PyHocaGUI_DialogBoxListDesktops(wx.Dialog):
         self.cancelBtn = wx.Button(self, wx.ID_CANCEL, _(u'Cancel'), )
 
         self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnListClick, self.desktopList)
+        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnListDoubleClick, self.desktopList)
         self.Bind(wx.EVT_BUTTON, self.OnOk, self.okBtn)
         self.Bind(wx.EVT_BUTTON, self.OnRefreshDesktopList, self.refreshBtn)
         self.Bind(wx.EVT_BUTTON, self.OnCancel, self.cancelBtn)
@@ -179,6 +179,17 @@ class PyHocaGUI_DialogBoxListDesktops(wx.Dialog):
                 user = desktop.split('@')[1]
                 self.add_item(user, display)
 
+    def OnListDoubleClick(self, evt):
+        """\
+        On double click select item and auto-click Ok button.
+
+        @param evt: event
+        @type evt: C{obj}
+
+        """
+        self.OnListClick(evt)
+        self.OnOk(evt)
+
     def OnListClick(self, evt):
         """\
         Enable the Connect button only if a list item got clicked.
