@@ -42,7 +42,7 @@ import wx
 import os
 
 # PyHoca-GUI modules
-# ... NONE ...
+import passphrase
 
 if os.environ.has_key('DESKTOP_SESSION'):
     WINDOW_MANAGER = os.environ['DESKTOP_SESSION']
@@ -54,7 +54,7 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
     Logon window for L{PyHocaGUI}.
 
     """
-    def __init__(self, _PyHocaGUI, profile_name, caller=None, sshproxy_auth=False): 
+    def __init__(self, _PyHocaGUI, profile_name, caller=None, passphrase=None, sshproxy_passphrase=None, sshproxy_auth=False, twofactor_auth=False, sshproxy_twofactor_auth=False): 
         """\
         Logon window (constructor)
 
@@ -66,6 +66,10 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
         @type caller: C{None}
         @param sshproxy_auth: use (dual) SSH proxy authentication
         @type sshproxy_auth: C{bool}
+        @param twofactor_auth: use two-factor authentication for X2Go Server
+        @type twofactor_auth: C{bool}
+        @param sshproxy_twofactor_auth: use two-factor authentication for SSH proxy
+        @type sshproxy_twofactor_auth: C{bool}
 
         """
         self._PyHocaGUI = _PyHocaGUI
@@ -73,6 +77,8 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
         self._pyhoca_logger('password dialog box started', loglevel=x2go.loglevel_INFO, )
 
         self.sshproxy_auth = sshproxy_auth
+        self.twofactor_auth = twofactor_auth
+        self.sshproxy_twofactor_auth = sshproxy_twofactor_auth
 
         self.current_profile_name = profile_name
         self.current_profile_config = self._PyHocaGUI.session_profiles.get_profile_config(profile_name)
@@ -86,6 +92,8 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
             self.SetTitle(_(u'%s (via %s)') % (profile_name, self.current_profile_config['sshproxyhost']))
 
         self.password = None
+        self.passphrase = passphrase
+        self.sshproxy_passphrase = sshproxy_passphrase
 
         self.userLbl = wx.StaticText(self, wx.ID_ANY, _(u'Username')+':', size=(-1, -1)) 
         self.userTxt = wx.TextCtrl(self, wx.ID_ANY, '', style=wx.TE_PROCESS_ENTER, size=(120, -1)) 
@@ -274,23 +282,25 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
                 return
 
         if self.sshproxy_auth and (not self.sshproxy_started):
-            force_password_auth=False
+            force_password_auth = False
             sshproxy_force_password_auth = True
         else:
-            force_password_auth=True
+            force_password_auth = True 
             sshproxy_force_password_auth = True
 
         wx.BeginBusyCursor()
         session_uuid = self._PyHocaGUI._X2GoClient__client_registered_sessions_of_profile_name(self.current_profile_name)[0]
         try:
-            self._PyHocaGUI._X2GoClient__connect_session(session_uuid, 
-                                                         username=username, 
-                                                         password=password, 
-                                                         force_password_auth=force_password_auth,
+            self._PyHocaGUI._X2GoClient__connect_session(session_uuid,
+                                                         username=username,
+                                                         password=password,
+                                                         passphrase=self.passphrase,
+                                                         force_password_auth=(force_password_auth and not self.twofactor_auth),
                                                          add_to_known_hosts=self._PyHocaGUI.add_to_known_hosts,
                                                          sshproxy_user=sshproxy_user,
                                                          sshproxy_password=sshproxy_password,
-                                                         sshproxy_force_password_auth=sshproxy_force_password_auth,
+                                                         sshproxy_passphrase=self.sshproxy_passphrase,
+                                                         sshproxy_force_password_auth=(sshproxy_force_password_auth and not self.sshproxy_twofactor_auth),
                                                         )
             if not self._PyHocaGUI._X2GoClient__server_valid_x2gouser(session_uuid):
                 self._PyHocaGUI.notifier.prepare('AUTH_%s' % self.current_profile_name,
@@ -312,6 +322,17 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
                     _sp.write_user_config = True
                     _sp.write()
 
+        except x2go.X2GoSSHProxyPasswordRequiredException:
+            key_filename = None
+            try:
+                if not self._PyHocaGUI._X2GoClient__get_session(session_uuid).sshproxy_params['sshproxy_look_for_keys']:
+                    key_filename = self._PyHocaGUI._X2GoClient__get_session(session_uuid).sshproxy_params['sshproxy_key_filename']
+            except KeyError:
+                pass
+            self._pyhoca_logger('SSH private key file (for SSH proxy) is encrypted and requires a passphrase', loglevel=x2go.log.loglevel_INFO, )
+            _passphrase_window = passphrase.PyHocaGUI_DialogBoxPassphrase(self._PyHocaGUI, self.current_profile_name, caller=self, password=password, sshproxy_auth=True, key_filename=key_filename)
+            self._PyHocaGUI._logon_windows[self.current_profile_name] = _passphrase_window
+
         except x2go.AuthenticationException:
             if self.sshproxy_auth and (not self.sshproxy_started):
                 try: wx.EndBusyCursor()
@@ -412,6 +433,29 @@ class PyHocaGUI_DialogBoxPassword(wx.Dialog):
 
         except x2go.SSHException, e:
 
+            if str(e).startswith('Two-factor authentication requires a password'):
+                if self.sshproxy_auth and (not self.sshproxy_started):
+                    try: wx.EndBusyCursor()
+                    except: pass
+                    self.sshproxy_started = True
+                    self.headerLbl.Enable(True)
+                    self.userLbl.Enable(True)
+                    self.userTxt.Enable(True)
+                    self.passwordLbl.Enable(True)
+                    self.passwordTxt.Enable(True)
+                    self.passwordTxt.SetFocus()
+                    self.loginBtn.Enable(True)
+                    self.loginBtn.SetDefault()
+                    self.cancelBtn.Enable(True)
+                    self.sshProxyHeaderLbl.Enable(False)
+                    self.sshProxyUserLbl.Enable(False)
+                    self.sshProxyUserTxt.Enable(False)
+                    self.sshProxyPasswordLbl.Enable(False)
+                    self.sshProxyPasswordTxt.Enable(False)
+                    self.sshProxyLoginBtn.Enable(False)
+                    self.sshProxyLoginBtn.SetLabel(_(u'SSH tunnel started'))
+                    return
+
             if str(e).startswith('Host key for server ') and str(e).endswith(' does not match!'):
                 errmsg = _('Host key verification failed. The X2Go server may have been compromised.\n\nIt is also possible that the host key has just been changed.\n\nHowever, for security reasons the connection will not be established!!!')
             else:
