@@ -126,16 +126,20 @@ class libnotify_NotifierPopup(object):
             except KeyError:
                 pass
 
-        icon = 'file://%s/PyHoca/32x32/%s.png' % (_icons_location, icon)
+        if icon:
+            icon = 'file://%s/PyHoca/32x32/%s.png' % (_icons_location, icon)
 
-        self._pyhoca_logger('[%s] %s' % (title.encode(utils.get_encoding()), text.encode(utils.get_encoding())), loglevel=log.loglevel_NOTICE)
+        if title and text:
+            self._pyhoca_logger('[%s] %s' % (title.encode(utils.get_encoding()), text.encode(utils.get_encoding())), loglevel=log.loglevel_NOTICE)
 
-        if not self._PyHocaGUI.disable_notifications:
-            n = pynotify.Notification(title, text, icon)
-            n.set_urgency(pynotify.URGENCY_NORMAL)
-            n.set_timeout(timeout)
-            if not n.show():
-                raise PyHocaNotificationException('could not notify user')
+        try:
+            if not self._PyHocaGUI.disable_notifications and title and text:
+                n = pynotify.Notification(title, text, icon)
+                n.set_urgency(pynotify.URGENCY_NORMAL)
+                n.set_timeout(timeout)
+                n.show()
+        except:
+            pass
 
     def Close(self):
         """\
@@ -268,7 +272,7 @@ class showballoon_NotifierPopup(object):
            icon = wx.ICON_INFORMATION
 
         try:
-            if not self._PyHocaGUI.disable_notifications:
+            if not self._PyHocaGUI.disable_notifications and title and text:
                 # you will need wxPython >= 2.9 for this
                 self._PyHocaGUI.taskbar.ShowBalloon(
                     title,
@@ -279,14 +283,15 @@ class showballoon_NotifierPopup(object):
         except:
             pass
 
-        # on Windows some error messages are already encoded, some are not, depending from which module they come
-        try: _title = title.encode(utils.get_encoding())
-        except: _title = title
-        try: _text = text.encode(utils.get_encoding())
-        except: _text = text
+        if title and text:
+            # on Windows some error messages are already encoded, some are not, depending from which module they come
+            try: _title = title.encode(utils.get_encoding())
+            except: _title = title
+            try: _text = text.encode(utils.get_encoding())
+            except: _text = text
 
-        try: self._pyhoca_logger('['+_title+'] '+_text, loglevel=log.loglevel_NOTICE)
-        except UnicodeDecodeError: self._pyhoca_logger('Unicode error occurred while rendering a log message...', loglevel=log.loglevel_WARN)
+            try: self._pyhoca_logger('['+_title+'] '+_text, loglevel=log.loglevel_NOTICE)
+            except UnicodeDecodeError: self._pyhoca_logger('Unicode error occurred while rendering a log message...', loglevel=log.loglevel_WARN)
 
     def Close(self):
         """\
@@ -421,7 +426,7 @@ class notificationmessage_NotifierPopup(object):
            icon = wx.ICON_INFORMATION
 
         try:
-            if not self._PyHocaGUI.disable_notifications:
+            if not self._PyHocaGUI.disable_notifications and title and text:
                 # you will need wxPython >= 2.9 for this
                 _notification_msg = wx.NotificationMessage()
                 _notification_msg.SetTitle(title)
@@ -434,14 +439,15 @@ class notificationmessage_NotifierPopup(object):
             # wx.NotificationMessage class
             pass
 
-        # on Windows some error messages are already encoded, some are not, depending from which module they come
-        try: _title = title.encode(utils.get_encoding())
-        except: _title = title
-        try: _text = text.encode(utils.get_encoding())
-        except: _text = text
+        if title and text:
+            # on Windows some error messages are already encoded, some are not, depending from which module they come
+            try: _title = title.encode(utils.get_encoding())
+            except: _title = title
+            try: _text = text.encode(utils.get_encoding())
+            except: _text = text
 
-        try: self._pyhoca_logger('['+_title+'] '+_text, loglevel=log.loglevel_NOTICE)
-        except UnicodeDecodeError: self._pyhoca_logger('Unicode error occurred while rendering a log message...', loglevel=log.loglevel_WARN)
+            try: self._pyhoca_logger('['+_title+'] '+_text, loglevel=log.loglevel_NOTICE)
+            except UnicodeDecodeError: self._pyhoca_logger('Unicode error occurred while rendering a log message...', loglevel=log.loglevel_WARN)
 
     def Close(self):
         """\
