@@ -47,6 +47,9 @@ class PyHocaGUI_SplashScreen(wx.SplashScreen):
         @type splash_image: C{str}
 
         """
+        if splash_image:
+            splash_image = os.path.expanduser(splash_image)
+
         if splash_image and os.path.basename(splash_image) == splash_image:
             splash_image = os.path.join(basepath.images_basepath, splash_image)
 
