@@ -71,7 +71,14 @@ def MakeIcon(icon_name, fallback_name='pyhoca-trayicon'):
     if icon_name is None:
         icon_name = fallback_name
 
-    icon_file = '%s/PyHoca/%s/%s.png' % (_icons_location, icon_size, icon_name)
+    icon_name = os.path.expanduser(icon_name)
+    if not icon_name.lower().endswith('.png'):
+        icon_name = '%s.png' % icon_name
+
+    icon_file = icon_name
+    if not (os.path.isfile(str(icon_file)) or os.path.islink(str(icon_file))):
+        icon_file = '%s/PyHoca/%s/%s' % (_icons_location, icon_size, icon_name)
+
     if not (os.path.isfile(str(icon_file)) or os.path.islink(str(icon_file))):
         icon_file = '%s/PyHoca/%s/%s.png' % (_icons_location, icon_size, fallback_name)
 
